import os
import sys

sys.path.insert(0, os.path.abspath('../../..'))
sys.path.insert(0, os.path.abspath('../../..'))
print(f"TEST: {os.path.abspath('../../..')}")


project = 'py-graspi'
copyright = '2024, Olga Wodo, Michael Leung, Wenqi Zheng, Qi Pan, Jerry Zhou, Kevin Martinez'
author = 'Michael Leung, Wenqi Zheng, Qi Pan, Jerry Zhou, Kevin Martinez'

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.napoleon',
    'sphinx.ext.viewcode',
    'sphinx.ext.autosummary',
    'sphinxcontrib.details.directive',
]

autosummary_generate = True

templates_path = ['_templates']


exclude_patterns = [
    '**/setup.py',
    'api/setup.rst',
    '**/test.py',
    'api/graspi_igraph.tests.rst',
    '_build',
    'Thumbs.db', '.DS_Store',
]

html_theme = 'sphinx_rtd_theme'
html_static_path = ['_static']
html_baseurl = 'https://owodolab.github.io/py-graspi/'

autodoc_default_options = {
    'members': True,
    'undoc-members': True,
    'private-members': True,
    'special-members': '__init__',
    'inherited-members': True,
    'show-inheritance': True,
}

autodoc_mock_imports = ["matplotlib", "mpl_toolkits.mplot3d"]




import sys
from logging import DEBUG

from PIL import Image
import numpy as np
DEBUG = False

def img_to_txt(imageFile, resizeFactor, threshold=127.5):
    """
        Converts an image file into a .txt file, dimensions are multiplied by resizeFactor.

        Args:
            imageFile (string): path of the image file.
            resizeFactor (float): resizeFactor used to multiply X and Y dimensions of imageFile to either truncate or enlarge.
            threshold (float): threshold used for deciding color intensity of what is considered white or black in numpy array of image

        Returns:
            bool_array (numpy Array): numpy array of boolean array which signal if index should be black or white.
            *not really needed is used in function to convert and output resized .txt file of image into the resized directory.
        """
    # Open image file
    image = Image.open(imageFile).convert('L')  # Convert image to grayscale
    # Turn image into a numpy array
    numpyData = np.asarray(image)

    # Initialize x and y axis of image
    dimY = numpyData.shape[0]
    dimX = numpyData.shape[1]

    # Resize original image into a new image file
    partOutfile = imageFile[18:-4]
    new_image = image.resize((int(dimX * resizeFactor), int(dimY * resizeFactor)))
    new_image.save(f"resized/resized_{partOutfile}_{resizeFactor}x.png")
    image2 = Image.open(f"resized/resized_{partOutfile}_{resizeFactor}x.png").convert('L')
    if DEBUG:
        print(image2.format)
        print(image2.size)
        print(image2.mode)

    # Turn new image into an array and find the Rows and Columns of new image
    numpyData2 = np.asarray(image2)
    dimX = numpyData2.shape[1]
    dimY = numpyData2.shape[0]

    # Convert to boolean numpy array using the threshold
    bool_array = numpyData2 > threshold
    print(bool_array)
    # Set up output file for new txt file that will be created from the newly shrunken image
    outfile = f"resized/resized_{partOutfile}_{resizeFactor}x.txt"
    with open(outfile, "w") as file:
        file.write(f"{dimX} {dimY}\n")

        # Loop through new image array and write out into the new file
        for row in bool_array:
            file.write(" ".join(['1' if col else '0' for col in row]) + "\n")

    return bool_array

def main():
    input_file = sys.argv[1]
    resize_factor = sys.argv[2]
    resize_factor = float(resize_factor)
    img_to_txt(input_file, resize_factor)

if __name__ == "__main__":
    main()


import os
import tortuosity_histogram as t

import src.graph as ig
import src.descriptors as ds
import fpdf
import numpy as np
from PIL import Image, ImageOps
import webbrowser
import argparse
import matplotlib.pyplot as plt
import math



current_dir = os.getcwd()
# data_path = f"{current_dir}/py_graspi/data/"
# descriptors_path = f"{current_dir}/py_graspi/descriptors/"
# image_path = f"{current_dir}/py_graspi/images/"
# hist_path = f"{current_dir}/py_graspi/histograms/"
# results_path = f"{current_dir}/py_graspi/results/"
parent_dir = os.path.dirname(current_dir)
data_path = f"{parent_dir}/data/data/"
descriptors_path = f"{parent_dir}/data/descriptors/"
image_path = f"{parent_dir}/data/images/"
hist_path = f"{parent_dir}/data/histograms/"
results_path = f"{parent_dir}/data/results/"

# data_path = f"{parent_dir}/data/data/"
# descriptors_path = f"{parent_dir}/data/descriptors/"
# expected_distances_path = f"{parent_dir}/data/distances/"
# results_path = f"{parent_dir}/data/comparisons/"

test_files = [os.path.splitext(file)[0] for file in os.listdir(data_path) if os.path.splitext(file)[0].count("_") == 3]
epsilon = 1e-5

"""
Generates the black and white images of each morphology.
"""
def generate_image(filename):
    file_path = data_path + filename + ".txt"
    matrix = []
    with open(file_path, "r") as f:
        lines = f.readlines()
        for line in lines[1:]:
            row = []
            line = line.strip().split(" ")
            for char in line:
                row.append(int(char))
            matrix.append(row)
    matrix_array = np.array(matrix, dtype=np.uint8)
    image = Image.fromarray(matrix_array * 255, mode="L")
    bw_image = image.convert("1")
    outline_image = ImageOps.expand(bw_image, border=1, fill="black")
    outline_image.save(image_path + filename + ".png")

"""
Generates the histograms of each descriptor data for distances and tortuosity.
"""
def generate_histogram(data, bins, filename, title, labelX, labelY, color, start, step, stop):
    if title == "Path Balance":
        plt.hist(data, color=color, bins=bins, label=["White pixels to bottom","Black pixels to top"])
        plt.legend(fontsize=20)
    else:
        plt.hist(data, color=color, bins=bins)
    plt.title(title, fontsize=20)
    plt.xlabel(labelX, fontsize=20)
    plt.ylabel(labelY, fontsize=20)
    plt.xlim(start, stop)
    plt.xticks(ticks=np.arange(start=start, step=step, stop=stop), rotation=90, fontsize=20)
    plt.yticks(fontsize=20)
    plt.tight_layout()
    plt.savefig(hist_path + filename + ".png", format="png")
    plt.close()
    return hist_path + filename + ".png"

"""
Main function that generates the txt and pdf files of the report.
"""
def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("file_type", choices=["txt", "pdf"])
    args = parser.parse_args()

    """
    Checks if the user wants to generate the txt files or the pdf.
    """
    if args.file_type == "txt":
        PDF = False
    else:
        PDF = True

    pdf = None

    if PDF:
        pdf = fpdf.FPDF()
        pdf.set_font("Arial", size=10)
        print("Generating PDF")

    print("Generating Text Files")

    """
    Generates the txt files and pdf for each of the 33 morphology descriptors.
    """
    for test_file in test_files:
        print(f"Executing {test_file}")
        if PDF:
            pdf.add_page()

        if PDF:
            pdf.cell(200, 8, txt=f"Morphology: {test_file}", ln=True, align="L")

        if PDF:
            generate_image(test_file)
            image_file = image_path + test_file + ".png"
            pdf.image(image_file, h=15, w=60)

        """
        Generates the graph for each of the morphology descriptors
        """
        graphData = ig.generateGraph(

            data_path + test_file + ".txt")
        print(f"{test_file} Graph Generated")

        """
        Checks if txt files already exists. 
        If they exist and the user requests the pdf, it will only regenerate the pdf.
        txt must be selected to regenerate the txt files.
        """
        if os.path.exists(results_path + "descriptors-" + test_file + ".txt") and PDF:
            with open(results_path + "descriptors-" + test_file + ".txt", "r") as txt:
                next(txt)
                for line in txt.readlines():
                    pdf.cell(40, 8, txt=line, ln=True, align="L")
        else:
            stats = ds.descriptors(graphData, test_file)

            print(f"{test_file} Descriptors Generated")
            with open(results_path + "descriptors-" + test_file + ".txt", "w") as txt:
                txt.write(f"Morphology: {test_file}\n")

                for stat in stats:
                    txt.write(f"{stat} {stats[stat]}\n")
                    if PDF:
                        pdf.cell(40, 8, txt=f"{stat} {stats[stat]}", ln=True, align="L")

                print(f"{test_file} Text File Generated")

        """
        Generates and adds in the descriptor histograms to the pdf
        """
        if PDF:
            with open(data_path + test_file + "_DistancesWhiteToBlue.txt", "r") as f:
                data1 = [float(line.strip()) for line in f if not math.isinf(float(line.strip()))]
                hist1 = generate_histogram([data1], 25, test_file + "1", "Distance from A to Ca", "Distance","Instances", "Blue", 0, 50, 250)
                pdf.image(hist1, x=80, y=10, w=60)

            with open(data_path + test_file + "_DistanceBlackToRed.txt", "r") as f:
                data2 = [float(line.strip()) for line in f if not math.isinf(float(line.strip()))]
                hist2 = generate_histogram([data2], 20, test_file + "2", "Distance from D to Am", "Distance","Instances", "Red", 0, 20, 200)
                pdf.image(hist2, x=142, y=10, w=60)

            hist3 = generate_histogram([data1, data2], 25, test_file + "3", "Path Balance", "Distance", "Instances",["Blue", "Red"], 0, 50, 250)
            pdf.image(hist3, x=80, y=60, w=60)

            with open(data_path + test_file + "_DistanceBlackToGreen.txt", "r") as f:
                data4 = [float(line.strip()) for line in f if not math.isinf(float(line.strip()))]
                hist4 = generate_histogram([data4], 12, test_file + "4", "Distance from D to Int", "Distance","Instances", "Green", 0, 10, 60)
                pdf.image(hist4, x=142, y=60, w=60)

            with open(data_path + test_file + "_TortuosityBlackToRed.txt", "r") as f:
                data5 = [float(line.strip()) for line in f if not math.isinf(float(line.strip()))]
                hist5 = generate_histogram([data5], 25, test_file + "5", "Tortuosity of D-paths to An", "Tortuosity","Instances", "Red", 1, 0.1, 1.5)
                pdf.image(hist5, x=80, y=110, w=60)

            with open(data_path + test_file + "_TortuosityWhiteToBlue.txt", "r") as f:
                data6 = [float(line.strip()) for line in f if not math.isinf(float(line.strip()))]
                hist6 = generate_histogram([data6], 25, test_file + "6", "Tortuosity of A-paths to Ca", "Tortuosity","Instances", "Blue", 1, 0.1, 1.5)
                pdf.image(hist6, x=142, y=110, w=60)

            """
            Generates the heat map of tortuosity between black and red, and white to blue
            """
            heat1 = t.find_BTR_tortuosity(graphData.graph, graphData.is_2D, test_file + ".txt", hist_path + test_file + "7.png", "Tortuosity of D-paths to An")
            pdf.image(hist_path + test_file + "7.png", x=80, y=160, w=60)

            heat2 = t.find_WTB_tortuosity(graphData.graph, graphData.is_2D, test_file + ".txt", hist_path + test_file + "8.png","Tortuosity of A-paths to Ca")
        
            pdf.image(hist_path + test_file + "8.png", x=142, y=160, w=60)


            print(f"{test_file} PDF Generated")


    print("Text Files Generated")

    """
    Outputs the generated pdf to the user.
    """
    if PDF:
        pdf.output(f"{parent_dir}/data/test_results.pdf")
        print("PDF Generated")
        webbrowser.open_new_tab(f"{parent_dir}/data/test_results.pdf")

if __name__ == "__main__":
    main()


import combined_without as ig
import new_descriptors as ds
import os
import time

current_dir = os.getcwd()
data_path = f"{current_dir}/graspi_igraph/data/"
descriptors_path = f"{current_dir}/graspi_igraph/descriptors/"
test_files = [os.path.splitext(file)[0] for file in os.listdir(data_path)]
epsilon=1e-5
for test_file in test_files:
    if 'morph' not in test_file:
        continue
    if '1x' not in test_file:
        continue
    # g = ig.generateGraph(data_path + test_file + ".txt")
    import time
    graph_start = time.time()
    g,is_2D, black_vertices, white_vertices, black_green, black_interface_red, white_interface_blue, dim, interface_edge_comp_paths, shortest_path_to_red, shortest_path_to_blue, CT_n_D_adj_An, CT_n_A_adj_Ca= ig.generateGraph(data_path + test_file + ".txt")
    graph_end = time.time()
    stats = ds.descriptors(g, test_file, black_vertices, white_vertices, black_green, black_interface_red, white_interface_blue, dim, interface_edge_comp_paths, shortest_path_to_red, shortest_path_to_blue, CT_n_D_adj_An, CT_n_A_adj_Ca)
    #ig.visual2D(g, 'graph')

    print(f"{test_file} Results")

    with open(descriptors_path + "descriptors." + test_file + ".log") as f:
        for line in f:
            stat = line.strip().split(" ")
            try:
                # if stats.get(stat[0], -1) == int(stat[1]):
                if abs(stats.get(stat[0], -1) - float(stat[1])) < epsilon:
                    print(f"{stat[0]} passed")
                elif stats.get(stat[0], -1) != -1 and stats.get(stat[0], -1) != int(stat[1]):
                    print(f"{stat[0]} failed - {stats.get(stat[0])} is not the same as expected {stat[1]}")
            except ValueError:
                if abs(stats.get(stat[0], -1) - float(stat[1])) < epsilon:
                    print(f"{stat[0]} passed")
                elif stats.get(stat[0], -1) != -1 and stats.get(stat[0], -1) != float(stat[1]):
                    print(f"{stat[0]} failed - {stats.get(stat[0])} is not the same as expected {stat[1]}")
    time = stats["time"]
    mem = stats["mem"]
    graph_time = graph_end-graph_start
    print(f"Total time to calculate graph: {graph_time} second(s)")
    print(f"Total time to calculate descriptors: {time} second(s)")
    print(f"Peak memory usage for descriptor calculation: {mem} bytes")
    print(stats)
    print("")




import os
import sys
import igraph
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../../src")))
sys.path.append(os.path.abspath('translations'))

import graph as ig

import tools.translations.img_to_txt as translate
import matplotlib.pyplot as plt
from matplotlib.widgets import Button
import numpy as np


def visualize(g):
    layout = g.layout('kk')
    fig, ax = plt.subplots(figsize=(200, 200))

    plot = igraph.plot(g,
                       target=ax,
                       layout=layout,
                       vertex_colors=g.vs["color"],
                       vertex_size=10,
                       margin=200)
    axcolor = 'lightgoldenrodyellow'

    ax_zoom_in = plt.axes([0.55, 0.05, 0.1, 0.075], facecolor=axcolor)
    ax_zoom_out = plt.axes([0.65, 0.05, 0.1, 0.075], facecolor=axcolor)
    ax_rotate = plt.axes([0.75, 0.05, 0.1, 0.075], facecolor=axcolor)
    ax_rotate_ccw = plt.axes([0.85, 0.05, 0.1, 0.075], facecolor=axcolor)

    button_zoom_in = Button(ax_zoom_in, label='Zoom In')
    button_zoom_out = Button(ax_zoom_out, label='Zoom Out')
    button_rotate = Button(ax_rotate, label='Rotate CW')
    button_rotate_opposite = Button(ax_rotate_ccw, label='Rotate CCW')

    current_angle = [0]

    button_zoom_in.on_clicked(lambda event: zoom_in(event, ax))
    button_zoom_out.on_clicked(lambda event: zoom_out(event, ax))
    button_rotate.on_clicked(lambda event: rotate(event, ax, g, layout, current_angle, 30))
    button_rotate_opposite.on_clicked(lambda event: rotate(event, ax, g, layout, current_angle, -30))

    plt.show()



def filter_black_vertices(graph):
    """
        Filters the graph by keeping only edges between vertices of the same color {black}.

        Args:
            graph (ig.Graph): The input graph.

        Returns:
            ig.Graph: The filtered graph.
        """
    edgeList = graph.get_edgelist()
    keptEdges = []

    # Checks edges and keeps only edges that connect to the same colored vertices
    for edge in edgeList:
        currentNode = edge[0]
        toNode = edge[1]
        if (graph.vs[currentNode]['color'] == 'black') and (graph.vs[toNode]['color'] == 'black'):
            keptEdges.append(edge)
        if ((graph.vs[currentNode]['color'] == 'red') and (graph.vs[toNode]['color'] == 'black')) or (
                graph.vs[currentNode]['color'] == 'black') and (graph.vs[toNode]['color'] == 'red'):
            keptEdges.append(edge)
        elif ((graph.vs[currentNode]['color'] == 'blue') and (graph.vs[toNode]['color'] == 'black')) or (
                graph.vs[currentNode]['color'] == 'black') and (graph.vs[toNode]['color'] == 'blue'):
            keptEdges.append(edge)
    filteredGraph = graph.subgraph_edges(keptEdges, delete_vertices=True)

    return filteredGraph


def filter_white_vertices(graph):
    """
        Filters the graph by keeping only edges between vertices of the same color {white}.

        Args:
            graph (ig.Graph): The input graph.

        Returns:
            ig.Graph: The filtered graph.
        """
    edgeList = graph.get_edgelist()
    keptEdges = []

    # Checks edges and keeps only edges that connect to the same colored vertices
    for edge in edgeList:
        currentNode = edge[0]
        toNode = edge[1]
        if (graph.vs[currentNode]['color'] == 'white') and (graph.vs[toNode]['color'] == 'white'):
            keptEdges.append(edge)
        if ((graph.vs[currentNode]['color'] == 'red') and (graph.vs[toNode]['color'] == 'white')) or (
                graph.vs[currentNode]['color'] == 'white') and (graph.vs[toNode]['color'] == 'red'):
            keptEdges.append(edge)
        elif ((graph.vs[currentNode]['color'] == 'blue') and (graph.vs[toNode]['color'] == 'white')) or (
                graph.vs[currentNode]['color'] == 'white') and (graph.vs[toNode]['color'] == 'blue'):
            keptEdges.append(edge)

    filteredGraph = graph.subgraph_edges(keptEdges, delete_vertices=True)

    return filteredGraph

def get_largest_subgraph(g):
    subgraphs = g.decompose()
    largest_subgraph = max(subgraphs, key=lambda sg:sg.vcount())
    return largest_subgraph
def zoom_in(event, ax):
    xlims = ax.get_xlim()
    ylims = ax.get_ylim()

    # Calculate the center of the plot
    x_center = (xlims[0] + xlims[1]) / 2
    y_center = (ylims[0] + ylims[1]) / 2

    # Define a more aggressive zoom factor
    factor = 0.5  # Change this value to zoom in more

    # Calculate new limits
    x_range = (xlims[1] - xlims[0]) * factor
    y_range = (ylims[1] - ylims[0]) * factor
    new_xlims = [x_center - x_range / 2, x_center + x_range / 2]
    new_ylims = [y_center - y_range / 2, y_center + y_range / 2]

    # Set new limits
    ax.set_xlim(new_xlims)
    ax.set_ylim(new_ylims)
    plt.draw()


def zoom_out(event, ax):
    xlims = ax.get_xlim()
    ylims = ax.get_ylim()

    # Calculate the center of the plot
    x_center = (xlims[0] + xlims[1]) / 2
    y_center = (ylims[0] + ylims[1]) / 2

    # Define a more aggressive zoom factor
    factor = 2  # Change this value to zoom in less

    # Calculate new limits
    x_range = (xlims[1] - xlims[0]) * factor
    y_range = (ylims[1] - ylims[0]) * factor
    new_xlims = [x_center - x_range / 2, x_center + x_range / 2]
    new_ylims = [y_center - y_range / 2, y_center + y_range / 2]

    # Set new limits
    ax.set_xlim(new_xlims)
    ax.set_ylim(new_ylims)
    plt.draw()


def rotate(event, ax, g, layout,current_angle, angle):
    # Preserve current zoom limits
    xlims = ax.get_xlim()
    ylims = ax.get_ylim()

    # Convert the angle to radians
    current_angle[0] += angle
    angle_rad = np.deg2rad(current_angle[0])

    # Get the original layout coordinates
    coords = np.array(layout.coords)

    # Rotate the coordinates
    cos_angle = np.cos(angle_rad)
    sin_angle = np.sin(angle_rad)
    rotation_matrix = np.array([[cos_angle, -sin_angle], [sin_angle, cos_angle]])
    rotated_coords = coords.dot(rotation_matrix)

    # Update the layout with the rotated coordinates
    new_layout = igraph.Layout(rotated_coords.tolist())

    # Clear the current plot and redraw the graph with the new layout
    ax.clear()
    plot = igraph.plot(g,
                       target=ax,
                       layout=new_layout,
                       vertex_colors=g.vs["color"],
                       vertex_size=10,
                       margin=20)

    # Reapply the previous zoom limits
    ax.set_xlim(xlims)
    ax.set_ylim(ylims)
    plt.draw()

def rotate_counterclockwise(event, ax, g, layout, current_angle, angle):
    # Preserve current zoom limits
    xlims = ax.get_xlim()
    ylims = ax.get_ylim()

    # Convert the angle to radians and update the current angle
    current_angle[0] += angle
    angle_rad = np.deg2rad(current_angle[0])

    # Get the original layout coordinates
    coords = np.array(layout.coords)

    # Rotate the coordinates counterclockwise
    cos_angle = np.cos(angle_rad)
    sin_angle = np.sin(angle_rad)
    rotation_matrix = np.array([[cos_angle, sin_angle], [-sin_angle, cos_angle]])
    rotated_coords = coords.dot(rotation_matrix)

    # Update the layout with the rotated coordinates
    new_layout = igraph.Layout(rotated_coords.tolist())

    # Clear the current plot and redraw the graph with the new layout
    ax.clear()
    plot = igraph.plot(g,
                       target=ax,
                       layout=new_layout,
                       vertex_colors=g.vs["color"],
                       vertex_size=10,
                       margin=20)

    # Reapply the previous zoom limits
    ax.set_xlim(xlims)
    ax.set_ylim(ylims)
    plt.draw()

def main():
    input_file = sys.argv[1]
    resize_factor = sys.argv[2]
    resize_factor = float(resize_factor)
    translate.img_to_txt(input_file,resize_factor)
    txt_filename = "resized/resized_" +input_file[18:-4]+ "_" + str(resize_factor) +"x.txt"
    print("creating graph")
    graph_data = ig.generateGraphAdj(txt_filename)


    print("graph created")
    print("filtering graph)")
    whiteFilteredGraph = filter_white_vertices(graph_data.graph)
    lWhiteSubGraph = get_largest_subgraph(whiteFilteredGraph)
    print("graph filtered")
    visualize(lWhiteSubGraph)
    print("filtering graph")
    blackFilteredGraph = filter_black_vertices(graph_data.graph)
    lBlackSubGraph = get_largest_subgraph(blackFilteredGraph)
    print("graph filtered")
    visualize(lBlackSubGraph)


if __name__ == "__main__":
    main()


import os
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../src")))
import graph as ig
import src.descriptors as d


graph_data = ig.generateGraphAdj(sys.argv[1])

dic = d.descriptors(graph_data, sys.argv[1])

for key, value in dic.items():
    print(key, value)



import numpy as np
import matplotlib.pyplot as plt
import os

current_dir = os.getcwd()

def find_coords(filename):
    # with open(f"{current_dir}/py_graspi/data/{filename}", "r") as file:
    with open(f"../data/data/{filename}", "r") as file:

        header = file.readline().split(' ')
        dimX, dimY = int(header[0]), int(header[1])
        if len(header) < 3:
            dimZ = 1
        else:
            if int(header[2]) == 0:
                dimZ = 1
            else:
                dimZ = int(header[2])

        if dimZ > 1:
            # dimZ = dimX * dimY
            is_2d = False
        coords = [dimX, dimY, dimZ]
    return coords

def filterGraph(graph):
    keptEdges = [edge for edge in graph.get_edgelist()
                 if graph.vs[edge[0]]['color'] == graph.vs[edge[1]]['color']
                 or 'red' in {graph.vs[edge[0]]['color'], graph.vs[edge[1]]['color']}
                 or 'blue' in {graph.vs[edge[0]]['color'], graph.vs[edge[1]]['color']}]

    return graph.subgraph_edges(keptEdges, delete_vertices=False)


def find_BTR_tortuosity(g, is_2d, filename, output, title):
    numVertices = g.vcount()
    redVertex = g.vcount() - 2
    blackToRedList = []
    filteredGraph = filterGraph(g)
    idOfPixelIn1DArray, tort = read_BTR_file_and_extract_numbers(filename)
    #Calculate vertex frequencies
    vertex_frequency = [0] * numVertices
    for i in range(len(idOfPixelIn1DArray)):
        vertex_frequency[idOfPixelIn1DArray[i]] = tort[i]

    vertex_frequency = vertex_frequency[:-3]
    dimX,dimY,dimZ = coords = find_coords(filename)
    data_2d = np.array(vertex_frequency).reshape(dimY, dimX)

    # Create the heatmap
    plt.rcParams.update({'font.size': 20})
    plt.title("Black to Red Tortuosity HeatMap")
    plt.imshow(data_2d, cmap='hot', interpolation='nearest')
    plt.colorbar()  # Add a colorbar to show the values
    plt.title(title, y=2, fontsize=20)
    plt.savefig(output)
    plt.close()

def find_WTB_tortuosity(g, is_2d, filename, output, title):
    numVertices = g.vcount()
    blueVertex = g.vcount() - 1
    whiteToBlueList = []
    filteredGraph = filterGraph(g)
    idOfPixelIn1DArray, tort = read_WTB_file_and_extract_numbers(filename)
    #Calculate vertex frequencies
    vertex_frequency = [0] * numVertices
    for i in range(len(idOfPixelIn1DArray)):
        vertex_frequency[idOfPixelIn1DArray[i]] = tort[i]

    vertex_frequency = vertex_frequency[:-3]
    dimX,dimY,dimZ = coords = find_coords(filename)
    data_2d = np.array(vertex_frequency).reshape(dimY, dimX)

    # Create the heatmap
    plt.rcParams.update({'font.size': 20})
    plt.title("White to Blue Tortuosity HeatMap")
    plt.imshow(data_2d, cmap='hot', interpolation='nearest')
    plt.colorbar()  # Add a colorbar to show the values
    plt.title(title, y=2, fontsize=20)
    plt.savefig(output)
    plt.close()

# Define the function to read the file and extract the numbers
def read_BTR_file_and_extract_numbers(base_filename):
    base_filename = base_filename[5:-4]
    file_path = f"../data/data/data_{base_filename}_IdTortuosityBlackToRed.txt"
    # file_path = f"{current_dir}/py_graspi/data/data_{base_filename}_IdTortuosityBlackToRed.txt"

    idOfPixelIn1DArray = []
    tort = []
    # Open the file in read mode
    with open(file_path, "r") as file:
        # Read each line in the file
        for line in file:
            # Split the line into a list of strings
            parts = line.split()
            # Extract the first and second numbers and convert them to appropriate types
            first_number = int(parts[0])
            second_number = float(parts[1])
            # Append the numbers to their respective lists
            idOfPixelIn1DArray.append(first_number)
            tort.append(second_number)

    return idOfPixelIn1DArray, tort

def read_WTB_file_and_extract_numbers(base_filename):
    base_filename = base_filename[5:-4]
    file_path = f"../data/data/data_{base_filename}_IdTortuosityWhiteToBlue.txt"
    # file_path = f"{current_dir}/py_graspi/data/data_{base_filename}_IdTortuosityWhiteToBlue.txt"

    idOfPixelIn1DArray = []
    tort = []
    # Open the file in read mode
    with open(file_path, "r") as file:
        # Read each line in the file
        for line in file:
            # Split the line into a list of strings
            parts = line.split()
            # Extract the first and second numbers and convert them to appropriate types
            first_number = int(parts[0])
            second_number = float(parts[1])
            # Append the numbers to their respective lists
            idOfPixelIn1DArray.append(first_number)
            tort.append(second_number)

    return idOfPixelIn1DArray, tort


from setuptools import setup, find_packages

setup(
    name = "py_graspi",
    author = "Wenqi Zheng",
    author_email = "wenqizhe@buffalo.edu",
    version = "0.1.1.7",
    description = "Utilize Python-igraph to produce similar functionality as GraSPI",
    packages = find_packages(),
    classifiers = ["Programming Language :: Python"],
    install_requires=[
        "igraph",
        "matplotlib",
        "numpy",
        "contourpy",
        "cycler",
        "fonttools",
        "kiwisolver",
        "packaging",
        "pillow",
        "psutil",
        "pyparsing",
        "python-dateutil",
        "six",
        "texttable",
        "fpdf"
    ],
    python_requires = ">=3.7"
    
)


import sys

import numpy as np

def translate_data(input_file):
    base_filename = input_file[4:-4]
    outputFileName = f"plt/{base_filename}.txt"
    with open(input_file, 'r') as file:
        # Read the file content
        line = file.readline()
        line = file.readline()
        line = line.split()

        #check if 2D of 3D by checking if "Z" exists in the .plt file
        if '"Z"' in line:
            line = file.readline().split()
            numOfVertices = int(line[1][2:-1])
            numOfEdges = int(line[2][2:-1])
            f = str(line[3][2:-1])
            et = str(line[4][3:])
            grid = []
            zCheckgrid = []
            line = file.readline().split()
            zLayer = 0.0
            gridInd = 0
            putIntoZ = True
            z_dim = 1
            for i in range(numOfVertices):
                if (float(line[0]) == 0):
                    grid.append([])
                    zCheckgrid.append([])
                    gridInd += 1
                    if (float(line[2]) != zLayer):
                        # grid.append([])
                        zCheckgrid[gridInd-1].append('\n')
                        z_dim += 1
                        zLayer += 1
                        putIntoZ = False
                numColorOfVertex = int(line[3])
                line = file.readline().split()
                grid[gridInd - 1].append(numColorOfVertex)
                if(putIntoZ):
                    zCheckgrid[gridInd - 1].append(0)
                else:
                    putIntoZ = True
            matrixOfGrid = np.array(grid)
            matrixCheckZ = np.array(zCheckgrid)
            y_dim, x_dim= matrixOfGrid.shape
            y_dim = int(y_dim/z_dim)

            outFile = open(outputFileName, "w")
            outFile.close()
            outFile = open(outputFileName, "a")
            outFile.write(f"{x_dim} {y_dim} {z_dim}")

            # Loop through the matrix
            for i in range(len(grid)):
                outFile.write("\n")
                for j in range(len(grid[i])):
                    # print(f"Element at ({i}, {j}): {grid[i][j]}")
                    if(matrixCheckZ[i][j] == '\n'):
                        outFile.write("\n")
                    outFile.write(f"{grid[i][j]} ")
        else:
            line = file.readline().split()
            numOfVertices = int(line[1][2:-1])
            numOfEdges = int(line[2][2:-1])
            f = str(line[3][2:-1])
            et = str(line[4][3:])
            grid = []
            line = file.readline().split()

            gridInd = 0
            for i in range(numOfVertices):
                if(float(line[0]) == 0):
                    grid.append([])
                    gridInd += 1
                numColorOfVertex = int(line[2])
                line= file.readline().split()
                grid[gridInd-1].append(numColorOfVertex)
            matrixOfGrid = np.array(grid)
            y_dim, x_dim = matrixOfGrid.shape
            outFile = open(outputFileName, "w")
            outFile.close()
            outFile = open(outputFileName, "a")
            outFile.write(f"{x_dim} {y_dim} 0")
            # Loop through the matrix
            for i in range(len(grid)):
                outFile.write("\n")
                for j in range(len(grid[i])):
                    # print(f"Element at ({i}, {j}): {grid[i][j]}")
                    outFile.write(f"{grid[i][j]} ")


def main():
    input_file = sys.argv[1]
    translate_data(input_file)

if __name__ == "__main__":
    main()

